@@ -6,7 +6,6 @@ use {
     },
     solana_cli_output::OutputFormat,
     solana_client::rpc_client::RpcClient,
-    solana_core::test_validator::TestValidator,
     solana_faucet::faucet::run_local_faucet,
     solana_sdk::{
         account_utils::StateMut,
@@ -17,6 +16,7 @@ use {
         signature::{Keypair, Signer},
     },
     solana_streamer::socket::SocketAddrSpace,
+    solana_test_validator::TestValidator,
     std::{env, fs::File, io::Read, path::PathBuf, str::FromStr},
 };
 
