@@ -9,7 +9,6 @@ use {
         blockhash_query::{self, BlockhashQuery},
         rpc_client::RpcClient,
     },
-    solana_core::test_validator::TestValidator,
     solana_faucet::faucet::run_local_faucet,
     solana_sdk::{
         account_utils::StateMut,
@@ -17,6 +16,7 @@ use {
         signature::{Keypair, NullSigner, Signer},
     },
     solana_streamer::socket::SocketAddrSpace,
+    solana_test_validator::TestValidator,
     solana_vote_program::vote_state::{VoteAuthorize, VoteState, VoteStateVersions},
 };
 
@@ -180,7 +180,8 @@ fn test_vote_authorize_and_withdraw() {
         fee_payer: 0,
     };
     process_command(&config).unwrap();
-    check_recent_balance(expected_balance - 100, &rpc_client, &vote_account_pubkey);
+    let expected_balance = expected_balance - 100;
+    check_recent_balance(expected_balance, &rpc_client, &vote_account_pubkey);
     check_recent_balance(100, &rpc_client, &destination_account);
 
     // Re-assign validator identity
@@ -199,6 +200,20 @@ fn test_vote_authorize_and_withdraw() {
         fee_payer: 0,
     };
     process_command(&config).unwrap();
+
+    // Close vote account
+    let destination_account = solana_sdk::pubkey::new_rand(); // Send withdrawal to new account to make balance check easy
+    config.signers = vec![&default_signer, &withdraw_authority];
+    config.command = CliCommand::CloseVoteAccount {
+        vote_account_pubkey,
+        withdraw_authority: 1,
+        destination_account_pubkey: destination_account,
+        memo: None,
+        fee_payer: 0,
+    };
+    process_command(&config).unwrap();
+    check_recent_balance(0, &rpc_client, &vote_account_pubkey);
+    check_recent_balance(expected_balance, &rpc_client, &destination_account);
 }
 
 #[test]
@@ -299,7 +314,7 @@ fn test_offline_vote_authorize_and_withdraw() {
 
     // Authorize vote account withdrawal to another signer, offline
     let withdraw_authority = Keypair::new();
-    let (blockhash, _) = rpc_client.get_recent_blockhash().unwrap();
+    let blockhash = rpc_client.get_latest_blockhash().unwrap();
     config_offline.command = CliCommand::VoteAuthorize {
         vote_account_pubkey,
         new_authorized_pubkey: withdraw_authority.pubkey(),
@@ -346,7 +361,7 @@ fn test_offline_vote_authorize_and_withdraw() {
 
     // Withdraw from vote account offline
     let destination_account = solana_sdk::pubkey::new_rand(); // Send withdrawal to new account to make balance check easy
-    let (blockhash, _) = rpc_client.get_recent_blockhash().unwrap();
+    let blockhash = rpc_client.get_latest_blockhash().unwrap();
     let fee_payer_null_signer = NullSigner::new(&default_signer.pubkey());
     config_offline.signers = vec![&fee_payer_null_signer, &withdraw_authority];
     config_offline.command = CliCommand::WithdrawFromVoteAccount {
@@ -388,7 +403,7 @@ fn test_offline_vote_authorize_and_withdraw() {
     check_recent_balance(100, &rpc_client, &destination_account);
 
     // Re-assign validator identity offline
-    let (blockhash, _) = rpc_client.get_recent_blockhash().unwrap();
+    let blockhash = rpc_client.get_latest_blockhash().unwrap();
     let new_identity_keypair = Keypair::new();
     let new_identity_null_signer = NullSigner::new(&new_identity_keypair.pubkey());
     config_offline.signers = vec![
